@@ -1,6 +1,6 @@
 /***********************************************************************************
 * Fancy Tasks: Plasmoid for fancy representing your tasks and launchers.
-* Copyright (C) 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
+* Copyright (C) 2009-2010 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
 ***********************************************************************************/
 
 #include "FancyTasksConfiguration.h"
+#include "FancyTasksActionDelegate.h"
 #include "FancyTasksApplet.h"
 #include "FancyTasksLauncher.h"
 
@@ -29,13 +30,17 @@
 #include <KIcon>
 #include <KLocale>
 #include <KFileDialog>
+#include <KMessageBox>
 #include <KServiceGroup>
 #include <KServiceTypeTrader>
 
 #include <taskmanager/taskmanager.h>
 #include <taskmanager/groupmanager.h>
 
-FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConfigDialog *parent) : QObject(parent),
+namespace FancyTasks
+{
+
+Configuration::Configuration(Applet *applet, KConfigDialog *parent) : QObject(parent),
     m_applet(applet)
 {
     KConfigGroup configuration = m_applet->config();
@@ -43,6 +48,7 @@ FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConf
     QWidget* generalWidget = new QWidget;
     QWidget* appearanceWidget = new QWidget;
     QWidget* arrangementWidget = new QWidget;
+    QWidget* actionsWidget = new QWidget;
     QWidget* findApplicationWidget = new QWidget;
     QAction* addLauncherApplicationAction = addLauncherMenu->addAction(KIcon("application-x-executable"), i18n("Add Application..."));
     QAction* addLauncherFromFileAction = addLauncherMenu->addAction(KIcon("inode-directory"), i18n("Add File or Directory..."));
@@ -58,6 +64,7 @@ FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConf
     m_generalUi.setupUi(generalWidget);
     m_appearanceUi.setupUi(appearanceWidget);
     m_arrangementUi.setupUi(arrangementWidget);
+    m_actionsUi.setupUi(actionsWidget);
     m_findApplicationUi.setupUi(findApplicationWidget);
 
     m_findApplicationDialog = new KDialog(parent);
@@ -81,29 +88,20 @@ FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConf
     m_generalUi.showOnlyCurrentDesktop->setChecked(configuration.readEntry("showOnlyCurrentDesktop", false));
     m_generalUi.showOnlyCurrentScreen->setChecked(configuration.readEntry("showOnlyCurrentScreen", false));
     m_generalUi.showOnlyMinimized->setChecked(configuration.readEntry("showOnlyMinimized", false));
+    m_generalUi.showOnlyTasksWithLaunchers->setChecked(configuration.readEntry("showOnlyTasksWithLaunchers", false));
     m_generalUi.connectJobsWithTasks->setChecked(configuration.readEntry("connectJobsWithTasks", false));
     m_generalUi.groupJobs->setChecked(configuration.readEntry("groupJobs", true));
 
-    m_generalUi.jobCloseMode->addItem(i18n("Instantly"), QVariant(FancyTasksApplet::InstantClose));
-    m_generalUi.jobCloseMode->addItem(i18n("After delay"), QVariant(FancyTasksApplet::DelayedClose));
-    m_generalUi.jobCloseMode->addItem(i18n("Manually"), QVariant(FancyTasksApplet::ManualClose));
-    m_generalUi.jobCloseMode->setCurrentIndex(m_generalUi.jobCloseMode->findData(QVariant(configuration.readEntry("jobCloseMode", static_cast<int>(FancyTasksApplet::DelayedClose)))));
+    m_generalUi.jobCloseMode->addItem(i18n("Instantly"), QVariant(InstantClose));
+    m_generalUi.jobCloseMode->addItem(i18n("After delay"), QVariant(DelayedClose));
+    m_generalUi.jobCloseMode->addItem(i18n("Manually"), QVariant(ManualClose));
+    m_generalUi.jobCloseMode->setCurrentIndex(m_generalUi.jobCloseMode->findData(QVariant(configuration.readEntry("jobCloseMode", static_cast<int>(DelayedClose)))));
 
     QStringList moveAnimationNames;
-    moveAnimationNames.append(i18n("None"));
-    moveAnimationNames.append(i18n("Zoom"));
-    moveAnimationNames.append(i18n("Jump"));
-    moveAnimationNames.append(i18n("Spotlight"));
-    moveAnimationNames.append(i18n("Glow"));
-    moveAnimationNames.append(i18n("Fade"));
+    moveAnimationNames << i18n("None") << i18n("Zoom") << i18n("Jump") << i18n("Spotlight") << i18n("Glow") << i18n("Fade");
 
-    QList<FancyTasksApplet::AnimationType> moveAnimationIds;
-    moveAnimationIds.append(FancyTasksApplet::NoAnimation);
-    moveAnimationIds.append(FancyTasksApplet::ZoomAnimation);
-    moveAnimationIds.append(FancyTasksApplet::JumpAnimation);
-    moveAnimationIds.append(FancyTasksApplet::SpotlightAnimation);
-    moveAnimationIds.append(FancyTasksApplet::GlowAnimation);
-    moveAnimationIds.append(FancyTasksApplet::FadeAnimation);
+    QList<AnimationType> moveAnimationIds;
+    moveAnimationIds << NoAnimation << ZoomAnimation << JumpAnimation << SpotlightAnimation << GlowAnimation << FadeAnimation;
 
     for (int i = 0; i < moveAnimationIds.count(); ++i)
     {
@@ -111,22 +109,10 @@ FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConf
     }
 
     QStringList iconAnimationNames;
-    iconAnimationNames.append(i18n("None"));
-    iconAnimationNames.append(i18n("Bounce"));
-    iconAnimationNames.append(i18n("Zoom"));
-    iconAnimationNames.append(i18n("Blink"));
-    iconAnimationNames.append(i18n("Spotlight"));
-    iconAnimationNames.append(i18n("Rotate"));
-    iconAnimationNames.append(i18n("Glow"));
+    iconAnimationNames << i18n("None") << i18n("Bounce") << i18n("Zoom") << i18n("Blink") << i18n("Spotlight") << i18n("Rotate") << i18n("Glow");
 
-    QList<FancyTasksApplet::AnimationType> iconAnimationIds;
-    iconAnimationIds.append(FancyTasksApplet::NoAnimation);
-    iconAnimationIds.append(FancyTasksApplet::BounceAnimation);
-    iconAnimationIds.append(FancyTasksApplet::ZoomAnimation);
-    iconAnimationIds.append(FancyTasksApplet::BlinkAnimation);
-    iconAnimationIds.append(FancyTasksApplet::SpotlightAnimation);
-    iconAnimationIds.append(FancyTasksApplet::RotateAnimation);
-    iconAnimationIds.append(FancyTasksApplet::GlowAnimation);
+    QList<AnimationType> iconAnimationIds;
+    iconAnimationIds << NoAnimation << BounceAnimation << ZoomAnimation << BlinkAnimation << SpotlightAnimation << RotateAnimation << GlowAnimation;
 
     for (int i = 0; i < iconAnimationIds.count(); ++i)
     {
@@ -134,26 +120,25 @@ FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConf
         m_appearanceUi.startupAnimation->addItem(iconAnimationNames.at(i), QVariant(iconAnimationIds.at(i)));
     }
 
-    m_appearanceUi.titleLabelMode->addItem(i18n("Never"), QVariant(FancyTasksApplet::NoLabel));
-    m_appearanceUi.titleLabelMode->addItem(i18n("On mouse-over"), QVariant(FancyTasksApplet::LabelOnMouseOver));
-    m_appearanceUi.titleLabelMode->addItem(i18n("For active icon"), QVariant(FancyTasksApplet::LabelForActiveIcon));
-    m_appearanceUi.titleLabelMode->addItem(i18n("Always"), QVariant(FancyTasksApplet::AlwaysShowLabel));
-    m_appearanceUi.titleLabelMode->setCurrentIndex(m_appearanceUi.titleLabelMode->findData(QVariant(configuration.readEntry("titleLabelMode", static_cast<int>(FancyTasksApplet::NoLabel)))));
+    m_appearanceUi.titleLabelMode->addItem(i18n("Never"), QVariant(NoLabel));
+    m_appearanceUi.titleLabelMode->addItem(i18n("On mouse-over"), QVariant(LabelOnMouseOver));
+    m_appearanceUi.titleLabelMode->addItem(i18n("For active icon"), QVariant(LabelForActiveIcon));
+    m_appearanceUi.titleLabelMode->addItem(i18n("Always"), QVariant(AlwaysShowLabel));
+    m_appearanceUi.titleLabelMode->setCurrentIndex(m_appearanceUi.titleLabelMode->findData(QVariant(configuration.readEntry("titleLabelMode", static_cast<int>(NoLabel)))));
 
-    m_appearanceUi.activeIconIndication->addItem(i18n("No indication"), QVariant(FancyTasksApplet::NoIndication));
-    m_appearanceUi.activeIconIndication->addItem(i18n("Zoom"), QVariant(FancyTasksApplet::ZoomIndication));
-    m_appearanceUi.activeIconIndication->addItem(i18n("Glow"), QVariant(FancyTasksApplet::GlowIndication));
-    m_appearanceUi.activeIconIndication->addItem(i18n("Fade"), QVariant(FancyTasksApplet::FadeIndication));
-    m_appearanceUi.activeIconIndication->setCurrentIndex(m_appearanceUi.activeIconIndication->findData(QVariant(configuration.readEntry("activeIconIndication", static_cast<int>(FancyTasksApplet::FadeIndication)))));
+    m_appearanceUi.activeIconIndication->addItem(i18n("No indication"), QVariant(NoIndication));
+    m_appearanceUi.activeIconIndication->addItem(i18n("Zoom"), QVariant(ZoomIndication));
+    m_appearanceUi.activeIconIndication->addItem(i18n("Glow"), QVariant(GlowIndication));
+    m_appearanceUi.activeIconIndication->addItem(i18n("Fade"), QVariant(FadeIndication));
+    m_appearanceUi.activeIconIndication->setCurrentIndex(m_appearanceUi.activeIconIndication->findData(QVariant(configuration.readEntry("activeIconIndication", static_cast<int>(FadeIndication)))));
 
     m_appearanceUi.useThumbnails->setChecked(configuration.readEntry("useThumbnails", false));
-    m_appearanceUi.useDefaultToolTips->setChecked(configuration.readEntry("useDefaultToolTips", false));
     m_appearanceUi.customBackgroundImage->setUrl(KUrl(configuration.readEntry("customBackgroundImage", QString())));
     m_appearanceUi.customBackgroundImage->setFilter("image/svg+xml image/svg+xml-compressed");
-    m_appearanceUi.moveAnimation->setCurrentIndex(moveAnimationIds.indexOf(static_cast<FancyTasksApplet::AnimationType>(configuration.readEntry("moveAnimation", static_cast<int>(FancyTasksApplet::ZoomAnimation)))));
+    m_appearanceUi.moveAnimation->setCurrentIndex(moveAnimationIds.indexOf(static_cast<AnimationType>(configuration.readEntry("moveAnimation", static_cast<int>(ZoomAnimation)))));
     m_appearanceUi.parabolicMoveAnimation->setChecked(configuration.readEntry("parabolicMoveAnimation", true));
-    m_appearanceUi.demandsAttentionAnimation->setCurrentIndex(iconAnimationIds.indexOf(static_cast<FancyTasksApplet::AnimationType>(configuration.readEntry("demandsAttentionAnimation", static_cast<int>(FancyTasksApplet::BlinkAnimation)))));
-    m_appearanceUi.startupAnimation->setCurrentIndex(iconAnimationIds.indexOf(static_cast<FancyTasksApplet::AnimationType>(configuration.readEntry("startupAnimation", static_cast<int>(FancyTasksApplet::BounceAnimation)))));
+    m_appearanceUi.demandsAttentionAnimation->setCurrentIndex(iconAnimationIds.indexOf(static_cast<AnimationType>(configuration.readEntry("demandsAttentionAnimation", static_cast<int>(BlinkAnimation)))));
+    m_appearanceUi.startupAnimation->setCurrentIndex(iconAnimationIds.indexOf(static_cast<AnimationType>(configuration.readEntry("startupAnimation", static_cast<int>(BounceAnimation)))));
 
     m_arrangementUi.removeButton->setIcon(KIcon("go-previous"));
     m_arrangementUi.addButton->setIcon(KIcon("go-next"));
@@ -191,7 +176,7 @@ FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConf
         }
         else
         {
-            FancyTasksLauncher *launcher = m_applet->launcherForUrl(KUrl(arrangement.at(i)));
+            Launcher *launcher = m_applet->launcherForUrl(KUrl(arrangement.at(i)));
 
             if (!launcher)
             {
@@ -205,11 +190,37 @@ FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConf
         m_arrangementUi.currentActionsListWidget->addItem(item);
     }
 
+    QStringList actionNames;
+    actionNames << i18n("Activate Item") << i18n("Activate Task") << i18n("Activate Launcher") << i18n("Show Item Menu") << i18n("Show Item Children List") << i18n("Show Item Windows") << i18n("Close Task");
+
+    QStringList actionOptions;
+    actionOptions << "activateItem" << "activateTask" << "activateLauncher" << "showItemMenu" << "showItemChildrenList" << "showItemWindows" << "closeTask";
+
+    QStringList actionDefaults;
+    actionDefaults << "left+" << QString('+') << "middle+" << QString('+') << QString('+') << "middle+shift" << "left+shift";
+
+    m_actionsUi.actionsTableWidget->setRowCount(actionNames.count());
+    m_actionsUi.actionsTableWidget->setItemDelegate(new ActionDelegate(this));
+
+    for (int i = 0; i < actionOptions.count(); ++i)
+    {
+        QTableWidgetItem *descriptionItem = new QTableWidgetItem(actionNames.at(i));
+        descriptionItem->setToolTip(actionNames.at(i));
+        descriptionItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
+
+        QTableWidgetItem *editingItem = new QTableWidgetItem(configuration.readEntry((actionOptions.at(i) + "Action"), actionDefaults.at(i)));
+        editingItem->setFlags(Qt::ItemIsEditable | Qt::ItemIsSelectable | Qt::ItemIsEnabled);
+
+        m_actionsUi.actionsTableWidget->setItem(i, 0, descriptionItem);
+        m_actionsUi.actionsTableWidget->setItem(i, 1, editingItem);
+    }
+
     moveAnimationTypeChanged(m_appearanceUi.moveAnimation->currentIndex());
 
     parent->addPage(generalWidget, i18n("General"), "go-home");
     parent->addPage(appearanceWidget, i18n("Appearance"), "preferences-desktop-theme");
     parent->addPage(arrangementWidget, i18n("Arrangement"), "format-list-unordered");
+    parent->addPage(actionsWidget, i18n("Actions"), "configure-shortcuts");
 
     connect(parent, SIGNAL(okClicked()), this, SLOT(accepted()));
     connect(m_appearanceUi.moveAnimation, SIGNAL(currentIndexChanged(int)), this, SLOT(moveAnimationTypeChanged(int)));
@@ -228,7 +239,7 @@ FancyTasksConfiguration::FancyTasksConfiguration(FancyTasksApplet *applet, KConf
     connect(m_findApplicationDialog, SIGNAL(finished()), this, SLOT(closeFindApplicationDialog()));
 }
 
-void FancyTasksConfiguration::accepted()
+void Configuration::accepted()
 {
     KConfigGroup configuration = m_applet->config();
     QStringList arrangement;
@@ -253,18 +264,26 @@ void FancyTasksConfiguration::accepted()
         }
     }
 
+    QStringList actionOptions;
+    actionOptions << "activateItem" << "activateTask" << "activateLauncher" << "showItemMenu" << "showItemChildrenList" << "showItemWindows" << "closeTask";
+
+    for (int i = 0; i < actionOptions.count(); ++i)
+    {
+        configuration.writeEntry((actionOptions.at(i) + "Action"), m_actionsUi.actionsTableWidget->item(i, 1)->data(Qt::EditRole).toString());
+    }
+
     configuration.writeEntry("moveAnimation", m_appearanceUi.moveAnimation->itemData(m_appearanceUi.moveAnimation->currentIndex()).toInt());
     configuration.writeEntry("parabolicMoveAnimation", m_appearanceUi.parabolicMoveAnimation->isChecked());
     configuration.writeEntry("demandsAttentionAnimation", m_appearanceUi.demandsAttentionAnimation->itemData(m_appearanceUi.demandsAttentionAnimation->currentIndex()).toInt());
     configuration.writeEntry("startupAnimation", m_appearanceUi.startupAnimation->itemData(m_appearanceUi.startupAnimation->currentIndex()).toInt());
     configuration.writeEntry("useThumbnails", m_appearanceUi.useThumbnails->isChecked());
-    configuration.writeEntry("useDefaultToolTips", m_appearanceUi.useDefaultToolTips->isChecked());
     configuration.writeEntry("activeIconIndication", m_appearanceUi.activeIconIndication->itemData(m_appearanceUi.activeIconIndication->currentIndex()).toInt());
     configuration.writeEntry("titleLabelMode", m_appearanceUi.titleLabelMode->itemData(m_appearanceUi.titleLabelMode->currentIndex()).toInt());
     configuration.writeEntry("customBackgroundImage", (m_appearanceUi.customBackgroundImage->url().isValid()?m_appearanceUi.customBackgroundImage->url().path():QString()));
     configuration.writeEntry("showOnlyCurrentDesktop", m_generalUi.showOnlyCurrentDesktop->isChecked());
     configuration.writeEntry("showOnlyCurrentScreen", m_generalUi.showOnlyCurrentScreen->isChecked());
     configuration.writeEntry("showOnlyMinimized", m_generalUi.showOnlyMinimized->isChecked());
+    configuration.writeEntry("showOnlyTasksWithLaunchers", m_generalUi.showOnlyTasksWithLaunchers->isChecked());
     configuration.writeEntry("connectJobsWithTasks", m_generalUi.connectJobsWithTasks->isChecked());
     configuration.writeEntry("groupJobs", m_generalUi.groupJobs->isChecked());
     configuration.writeEntry("groupingStrategy", m_generalUi.groupingStrategy->itemData(m_generalUi.groupingStrategy->currentIndex()).toInt());
@@ -275,12 +294,12 @@ void FancyTasksConfiguration::accepted()
     emit finished();
 }
 
-void FancyTasksConfiguration::moveAnimationTypeChanged(int option)
+void Configuration::moveAnimationTypeChanged(int option)
 {
-    m_appearanceUi.parabolicMoveAnimation->setEnabled(option != 0 && m_appearanceUi.moveAnimation->itemData(option).toInt() != static_cast<int>(FancyTasksApplet::JumpAnimation));
+    m_appearanceUi.parabolicMoveAnimation->setEnabled(option != 0 && m_appearanceUi.moveAnimation->itemData(option).toInt() != static_cast<int>(JumpAnimation));
 }
 
-void FancyTasksConfiguration::availableActionsCurrentItemChanged(int row)
+void Configuration::availableActionsCurrentItemChanged(int row)
 {
     m_arrangementUi.addButton->setEnabled(row >= 0);
 
@@ -290,7 +309,7 @@ void FancyTasksConfiguration::availableActionsCurrentItemChanged(int row)
     }
 }
 
-void FancyTasksConfiguration::currentActionsCurrentItemChanged(int row)
+void Configuration::currentActionsCurrentItemChanged(int row)
 {
     m_arrangementUi.removeButton->setEnabled(row >= 0);
     m_arrangementUi.moveUpButton->setEnabled(row > 0);
@@ -302,7 +321,7 @@ void FancyTasksConfiguration::currentActionsCurrentItemChanged(int row)
     }
 }
 
-void FancyTasksConfiguration::removeItem()
+void Configuration::removeItem()
 {
     if (m_arrangementUi.currentActionsListWidget->currentRow() >= 0)
     {
@@ -322,7 +341,7 @@ void FancyTasksConfiguration::removeItem()
     }
 }
 
-void FancyTasksConfiguration::addItem()
+void Configuration::addItem()
 {
     if (m_arrangementUi.availableActionsListWidget->currentRow() >= 0)
     {
@@ -339,7 +358,7 @@ void FancyTasksConfiguration::addItem()
     }
 }
 
-void FancyTasksConfiguration::moveUpItem()
+void Configuration::moveUpItem()
 {
     const int currentRow = m_arrangementUi.currentActionsListWidget->currentRow();
 
@@ -352,7 +371,7 @@ void FancyTasksConfiguration::moveUpItem()
     }
 }
 
-void FancyTasksConfiguration::moveDownItem()
+void Configuration::moveDownItem()
 {
     const int currentRow = m_arrangementUi.currentActionsListWidget->currentRow();
 
@@ -366,7 +385,7 @@ void FancyTasksConfiguration::moveDownItem()
     }
 }
 
-void FancyTasksConfiguration::addLauncher()
+void Configuration::addLauncher()
 {
     KFileDialog dialog(KUrl("~"), "", NULL);
     dialog.setWindowModality(Qt::NonModal);
@@ -376,7 +395,7 @@ void FancyTasksConfiguration::addLauncher()
 
     if (!dialog.selectedUrl().isEmpty())
     {
-        FancyTasksLauncher* launcher = new FancyTasksLauncher(dialog.selectedUrl(), m_applet);
+        Launcher* launcher = new Launcher(dialog.selectedUrl(), m_applet);
 
         QListWidgetItem *item = new QListWidgetItem(launcher->icon(), launcher->title(), m_arrangementUi.currentActionsListWidget);
         item->setToolTip(launcher->launcherUrl().pathOrUrl());
@@ -387,13 +406,21 @@ void FancyTasksConfiguration::addLauncher()
     }
 }
 
-void FancyTasksConfiguration::addLauncher(const QString &url)
+void Configuration::addLauncher(const QString &url)
 {
     if (!url.isEmpty())
     {
-        m_findApplicationDialog->close();
+        for (int i = 0; i < m_arrangementUi.currentActionsListWidget->count(); ++i)
+        {
+            if (m_arrangementUi.currentActionsListWidget->item(i)->toolTip() == url)
+            {
+                KMessageBox::sorry(static_cast<QWidget*>(parent()), i18n("Launcher with this URL was already added."));
 
-        FancyTasksLauncher* launcher = new FancyTasksLauncher(KUrl(url), m_applet);
+                return;
+            }
+        }
+
+        Launcher* launcher = new Launcher(KUrl(url), m_applet);
 
         QListWidgetItem *item = new QListWidgetItem(launcher->icon(), launcher->title(), m_arrangementUi.currentActionsListWidget);
         item->setToolTip(launcher->launcherUrl().pathOrUrl());
@@ -404,11 +431,11 @@ void FancyTasksConfiguration::addLauncher(const QString &url)
     }
 }
 
-void FancyTasksConfiguration::addMenu(QAction *action)
+void Configuration::addMenu(QAction *action)
 {
     if (!action->data().isNull())
     {
-        FancyTasksLauncher* launcher = new FancyTasksLauncher(KUrl("menu:" + action->data().toString()), m_applet);
+        Launcher* launcher = new Launcher(KUrl("menu:" + action->data().toString()), m_applet);
 
         QListWidgetItem *item = new QListWidgetItem(launcher->icon(), launcher->title(), m_arrangementUi.currentActionsListWidget);
         item->setToolTip(launcher->launcherUrl().pathOrUrl());
@@ -419,7 +446,7 @@ void FancyTasksConfiguration::addMenu(QAction *action)
     }
 }
 
-void FancyTasksConfiguration::setServiceMenu()
+void Configuration::setServiceMenu()
 {
     KMenu *menu = qobject_cast<KMenu*>(sender());
 
@@ -478,7 +505,7 @@ void FancyTasksConfiguration::setServiceMenu()
     }
 }
 
-void FancyTasksConfiguration::findApplication(const QString &query)
+void Configuration::findApplication(const QString &query)
 {
     for (int i = (m_findApplicationUi.resultsLayout->count() - 1); i >= 0; --i)
     {
@@ -488,8 +515,6 @@ void FancyTasksConfiguration::findApplication(const QString &query)
 
     if (query.length() < 3)
     {
-        m_findApplicationUi.scrollArea->setFixedHeight(0);
-
         m_findApplicationDialog->adjustSize();
 
         return;
@@ -497,56 +522,59 @@ void FancyTasksConfiguration::findApplication(const QString &query)
 
     KService::List services = KServiceTypeTrader::self()->query("Application", QString("exist Exec and ( (exist Keywords and '%1' ~subin Keywords) or (exist GenericName and '%1' ~~ GenericName) or (exist Name and '%1' ~~ Name) )").arg(query));
 
-    int amount = 0;
-
     if (!services.isEmpty())
     {
         foreach (const KService::Ptr &service, services)
         {
             if (!service->noDisplay() && service->property("NotShowIn", QVariant::String) != "KDE")
             {
-                FancyTasksLauncher* launcher = new FancyTasksLauncher(KUrl(service->entryPath()), m_applet);
+                Launcher* launcher = new Launcher(KUrl(service->entryPath()), m_applet);
 
                 QWidget* entryWidget = new QWidget(static_cast<QWidget*>(parent()));
                 QLabel* iconLabel = new QLabel(entryWidget);
                 QLabel* textLabel = new QLabel(QString("%1<br /><small>%3</small>").arg(launcher->title()).arg(launcher->description()), entryWidget);
-                QLabel* addLauncherLabel = new QLabel(QString("<a href=\"%1\" style=\"text-decoration:none;\">+</a>").arg(service->entryPath()), entryWidget);
 
                 iconLabel->setPixmap(launcher->icon().pixmap(32, 32));
 
                 textLabel->setFixedSize(240, 40);
 
-                addLauncherLabel->setToolTip(i18n("Add this application"));
-
                 QHBoxLayout* entryWidgetLayout = new QHBoxLayout(entryWidget);
                 entryWidgetLayout->addWidget(iconLabel);
                 entryWidgetLayout->addWidget(textLabel);
-                entryWidgetLayout->addWidget(addLauncherLabel);
                 entryWidgetLayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
 
                 entryWidget->setToolTip(QString("<b>%1</b><br /><i>%2</i>").arg(launcher->title()).arg(launcher->description()));
                 entryWidget->setLayout(entryWidgetLayout);
                 entryWidget->setFixedSize(300, 40);
+                entryWidget->setObjectName(service->entryPath());
+                entryWidget->installEventFilter(this);
+                entryWidget->setCursor(QCursor(Qt::PointingHandCursor));
 
                 m_findApplicationUi.resultsLayout->addWidget(entryWidget);
 
                 delete launcher;
-
-                ++amount;
-
-                connect(addLauncherLabel, SIGNAL(linkActivated(QString)), this, SLOT(addLauncher(QString)));
             }
         }
     }
 
-    m_findApplicationUi.scrollArea->setFixedHeight(((amount < 10)?amount:10) * 45);
-
     m_findApplicationDialog->adjustSize();
 }
 
-void FancyTasksConfiguration::closeFindApplicationDialog()
+void Configuration::closeFindApplicationDialog()
 {
     findApplication(QString());
 
     m_findApplicationUi.query->setText(QString());
 }
+
+bool Configuration::eventFilter(QObject *object, QEvent *event)
+{
+    if (event->type() == QEvent::MouseButtonPress)
+    {
+        addLauncher(object->objectName());
+    }
+
+    return QObject::eventFilter(object, event);
+}
+
+}
